@@ -1,4 +1,4 @@
-/* IBM Corp. 2013, 2015 */
+/* IBM Corp. 2013, 2016 */
 
 //_GNU_SOURCE used for getline and posix_memalign
 #define _GNU_SOURCE
@@ -15,11 +15,28 @@ FILE *qc_dbg_file;
 char *qc_dbg_dump_dir;
 int   qc_dbg_indent;
 char *qc_dbg_use_dump;
+int   qc_consistency_check_requested;
 static char	    *qc_dbg_file_name;
 static long	     qc_dbg_autodump;
 static unsigned int  qc_dbg_dump_idx;
+static iconv_t	     qc_cd = (iconv_t)-1;
 
 
+static void __attribute__((destructor)) qc_destructor() {
+	if (qc_cd != (iconv_t)-1)
+		iconv_close(qc_cd);
+}
+
+struct qc_handle *qc_get_cec_handle(struct qc_handle *hdl) {
+	return hdl ? hdl->root : hdl;
+}
+
+struct qc_handle *qc_get_lpar_handle(struct qc_handle *hdl) {
+	for (hdl = hdl->root; hdl != NULL && *(int *)(hdl->layer) != QC_LAYER_TYPE_LPAR; hdl = hdl->next);
+
+	return hdl;
+}
+
 /* Update dbg_level from environment variable */
 static void qc_update_dbg_level(void) {
 	char *s, *end;
@@ -79,7 +96,7 @@ static int qc_debug_file_init(void) {
 			close(fd);
 			goto out_err;
 		}
-		qc_debug(NULL, "This is qclib v1.1.0\n");
+		qc_debug(NULL, "This is qclib v1.2.0, level a7ecaf7, date 2016-06-03 09:04:14 +0200\n");
 	}
 
 	return 0;
@@ -153,8 +170,10 @@ static int qc_debug_init(void) {
 	qc_update_dbg_level();
 	if (qc_dbg_level > 0 && !qc_dbg_file) {
 		// open the log file - failure to do so is not a fatal error
-		if (qc_debug_file_init())
+		if (qc_debug_file_init()) {
+			qc_dbg_level = 0;
 			goto out_err;
+		}
 		qc_debug(NULL, "Log level set to %ld\n", qc_dbg_level);
 	}
 	if (qc_dbg_use_dump) {
@@ -187,12 +206,11 @@ static int qc_debug_init(void) {
 
 	return 0;
 
+
 out_err:
 	// Nothing we can do about this except to disable debug messages to prevent further damage
 	free(qc_dbg_dump_dir);
 	qc_dbg_dump_dir = NULL;
-	qc_dbg_level = 0;
-	qc_dbg_file = NULL;
 	free(path);
 
 	return rc;
@@ -222,7 +240,7 @@ void qc_mark_dump_incomplete(struct qc_handle *hdl, char *missing_component) {
 }
 
 /* Convert EBCDIC input to ASCII in place, removing trailing whitespace */
-int qc_ebcdic_to_ascii(struct qc_handle *hdl, iconv_t *cd, char *inbuf, size_t insz) {
+int qc_ebcdic_to_ascii(struct qc_handle *hdl, char *inbuf, size_t insz) {
 	char *outbuf, *outbuf_start, *inbuf_start = inbuf;
 	size_t len, outsz = insz, insz_orig, outsz_orig;
 	int rc = 0;
@@ -235,7 +253,7 @@ int qc_ebcdic_to_ascii(struct qc_handle *hdl, iconv_t *cd, char *inbuf, size_t i
 	outbuf = outbuf_start;
 	insz_orig = insz;
 	outsz_orig = outsz;
-	len = iconv(*cd, &inbuf, &insz, &outbuf, &outsz);
+	len = iconv(qc_cd, &inbuf, &insz, &outbuf, &outsz);
 	if (len == (size_t)(-1)) {
 		qc_debug(hdl, "Error: iconv conversion failed: %s\n", strerror(errno));
 		rc = -2;
@@ -312,21 +330,40 @@ int qc_get_num_layers(void *config, int *rc) {
 	return hdl->layer_no + 1;
 }
 
-#define ATTR_UNDEF	4096
-/** Verifies that a + b (+ c) <= d (c is optional) for the respective int-attributes holds true.
+#define ATTR_UNDEF	qc_layer_name
+/** Verifies that a + (b (+ c)) <= d (b and c are optional, where b being unset (==ATTR_UNDEF) implies c being unset, too)
+ *  for the respective int-attributes holds true.
  * @param equals If set, we verify using '=', not '<='
  */
 static int qc_verify(struct qc_handle *hdl, enum qc_attr_id a, enum qc_attr_id b, enum qc_attr_id c, enum qc_attr_id d, int equals) {
-	int *val_a, *val_b, *val_c, *val_d;
+	int *val_a, *val_b = NULL, *val_c = NULL, *val_d;
 
 	// We assume that non-presence of a value is due to...non-presence, as opposed to an error (since that would have been reported previously)
 	if ((val_a = qc_get_attr_value_int(hdl, a)) == NULL ||
-	    (val_b = qc_get_attr_value_int(hdl, b)) == NULL ||
 	    (val_d = qc_get_attr_value_int(hdl, d)) == NULL)
 		return 0;
-	if (c < ATTR_UNDEF) {
-		if ((val_c = qc_get_attr_value_int(hdl, c)) == NULL)
-			return 0;
+	if (b != ATTR_UNDEF && (val_b = qc_get_attr_value_int(hdl, b)) == NULL)
+		return 0;
+	if (c != ATTR_UNDEF && (val_c = qc_get_attr_value_int(hdl, c)) == NULL)
+		return 0;
+
+	if (b == ATTR_UNDEF) {
+		if ((equals && *val_a != *val_d) || (!equals && *val_a > *val_d)) {
+			qc_debug(hdl, "Warning: Consistency check '%s %s %s' failed at layer %d (%s/%s): %d %s %d\n",
+				qc_attr_id_to_char(hdl, a), (equals ? "=" : "<="), qc_attr_id_to_char(hdl, d),
+				hdl->layer_no, qc_get_attr_value_string(hdl, qc_layer_type), qc_get_attr_value_string(hdl, qc_layer_category),
+				*val_a, (equals ? "!=" : ">"), *val_d);
+			return 1;
+		}
+	} else if (c == ATTR_UNDEF) {
+		if ((equals && *val_a + *val_b != *val_d) || (!equals && *val_a + *val_b > *val_d)) {
+			qc_debug(hdl, "Warning: Consistency check '%s + %s %s %s' failed at layer %d (%s/%s): %d + %d %s %d\n",
+				qc_attr_id_to_char(hdl, a), qc_attr_id_to_char(hdl, b), (equals ? "=" : "<="), qc_attr_id_to_char(hdl, d),
+				hdl->layer_no, qc_get_attr_value_string(hdl, qc_layer_type), qc_get_attr_value_string(hdl, qc_layer_category),
+				*val_a, *val_b, (equals ? "!=" : ">"), *val_d);
+			return 2;
+		}
+	} else {
 		if ((equals && *val_a + *val_b + *val_c != *val_d) || (!equals && *val_a + *val_b + *val_c > *val_d)) {
 			qc_debug(hdl, "Warning: Consistency check '%s + %s + %s %s %s' failed at layer %d (%s/%s): %d + %d + %d %s %d\n",
 				qc_attr_id_to_char(hdl, a), qc_attr_id_to_char(hdl, b),	qc_attr_id_to_char(hdl, c), (equals ? "=" : "<="),
@@ -334,12 +371,6 @@ static int qc_verify(struct qc_handle *hdl, enum qc_attr_id a, enum qc_attr_id b
 				qc_get_attr_value_string(hdl, qc_layer_category), *val_a, *val_b, *val_c, (equals ? "!=" : ">"), *val_d);
 			return 1;
 		}
-	} else if ((equals && *val_a + *val_b != *val_d) || (!equals && *val_a + *val_b > *val_d)) {
-		qc_debug(hdl, "Warning: Consistency check '%s + %s %s %s' failed at layer %d (%s/%s): %d + %d %s %d\n",
-			qc_attr_id_to_char(hdl, a), qc_attr_id_to_char(hdl, b), (equals ? "=" : "<="), qc_attr_id_to_char(hdl, d),
-			hdl->layer_no, qc_get_attr_value_string(hdl, qc_layer_type), qc_get_attr_value_string(hdl, qc_layer_category),
-			*val_a, *val_b, (equals ? "!=" : ">"), *val_d);
-		return 2;
 	}
 
 	return 0;
@@ -350,6 +381,8 @@ static int qc_verify(struct qc_handle *hdl, enum qc_attr_id a, enum qc_attr_id b
 static int qc_consistency_check(struct qc_handle *hdl) {
 	int *etype, rc = 0;
 
+	if (!qc_consistency_check_requested)
+		return 0;
 	qc_debug(hdl, "Run consistency check\n");
 	qc_debug_indent_inc();
 	for (; hdl; hdl = hdl->next) {
@@ -360,42 +393,53 @@ static int qc_consistency_check(struct qc_handle *hdl) {
 
 		switch (*etype) {
 		case QC_LAYER_TYPE_CEC:
-			if ((rc = qc_verify(hdl, qc_num_cpu_dedicated,  qc_num_cpu_shared,  ATTR_UNDEF,	    qc_num_cpu_total, 0)) ||
-			    (rc = qc_verify(hdl, qc_num_cpu_configured, qc_num_cpu_standby, qc_num_cpu_reserved, qc_num_cpu_total, 0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_total,	qc_num_ifl_total,   ATTR_UNDEF,	    qc_num_cpu_total, 0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_dedicated,   qc_num_cp_shared,   ATTR_UNDEF,	    qc_num_cp_total,  1)) ||
-			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,  qc_num_ifl_shared,  ATTR_UNDEF,	    qc_num_ifl_total, 1)))
+			if ((rc = qc_verify(hdl, qc_num_cpu_dedicated,  qc_num_cpu_shared,	ATTR_UNDEF,	     qc_num_cpu_total, 0)) ||
+			    (rc = qc_verify(hdl, qc_num_cpu_configured, qc_num_cpu_standby,	qc_num_cpu_reserved, qc_num_cpu_total, 0)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_total,	qc_num_ifl_total,	ATTR_UNDEF,	     qc_num_cpu_total, 0)) ||
+			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,	qc_num_cp_dedicated,	ATTR_UNDEF,	     qc_num_cpu_dedicated, 1)) ||
+			    (rc = qc_verify(hdl, qc_num_ifl_shared,	qc_num_cp_shared,	ATTR_UNDEF,	     qc_num_cpu_shared, 1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_dedicated,   qc_num_cp_shared,	ATTR_UNDEF,	     qc_num_cp_total,  1)) ||
+			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,  qc_num_ifl_shared,	ATTR_UNDEF,	     qc_num_ifl_total, 1)))
 				goto out;
 			break;
 		case QC_LAYER_TYPE_LPAR:
-			// FIXME: First check disabled, since cpu_total only counts CPs which gives us false positives
-			if (//(rc = qc_verify(hdl, qc_num_cpu_dedicated, qc_num_cpu_shared, ATTR_UNDEF, qc_num_cpu_total, 0)) ||
+			if ((rc = qc_verify(hdl, qc_num_cpu_dedicated,  qc_num_cpu_shared,	ATTR_UNDEF,	    qc_num_cpu_total, 0)) ||
 			    (rc = qc_verify(hdl, qc_num_cpu_configured, qc_num_cpu_reserved,	qc_num_cpu_standby, qc_num_cpu_total, 0)) ||
 			    (rc = qc_verify(hdl, qc_num_cp_dedicated,   qc_num_cp_shared,	ATTR_UNDEF,	    qc_num_cp_total,  1)) ||
 			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,  qc_num_ifl_shared,	ATTR_UNDEF,	    qc_num_ifl_total, 1)))
 				goto out;
 			break;
 		case QC_LAYER_TYPE_ZVM_HYPERVISOR:
-			if ((rc = qc_verify(hdl, qc_num_cpu_dedicated, qc_num_cpu_shared,	ATTR_UNDEF, qc_num_cpu_total,	  0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_total,      qc_num_ifl_total,	ATTR_UNDEF, qc_num_cpu_total,	  0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_dedicated,  qc_num_ifl_dedicated,	ATTR_UNDEF, qc_num_cpu_dedicated, 0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_shared,     qc_num_ifl_shared,	ATTR_UNDEF, qc_num_cpu_shared,	  0)) ||
+			if ((rc = qc_verify(hdl, qc_num_cpu_dedicated, qc_num_cpu_shared,	ATTR_UNDEF, qc_num_cpu_total,	  1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_total,      qc_num_ifl_total,	ATTR_UNDEF, qc_num_cpu_total,	  1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_dedicated,  qc_num_ifl_dedicated,	ATTR_UNDEF, qc_num_cpu_dedicated, 1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_shared,     qc_num_ifl_shared,	ATTR_UNDEF, qc_num_cpu_shared,	  1)) ||
 			    (rc = qc_verify(hdl, qc_num_cp_dedicated,  qc_num_cp_shared,	ATTR_UNDEF, qc_num_cp_total,	  1)) ||
 			    (rc = qc_verify(hdl, qc_num_ifl_dedicated, qc_num_ifl_shared,	ATTR_UNDEF, qc_num_ifl_total,	  1)))
 				goto out;
 			break;
 		case QC_LAYER_TYPE_ZVM_GUEST:
-			if ((rc = qc_verify(hdl, qc_num_cpu_dedicated,  qc_num_cpu_shared,	ATTR_UNDEF,	    qc_num_cpu_total,	  0)) ||
-			    (rc = qc_verify(hdl, qc_num_cpu_configured, qc_num_cpu_reserved,	qc_num_cpu_standby, qc_num_cpu_total,	  0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_total,       qc_num_ifl_total,	ATTR_UNDEF,	    qc_num_cpu_total,	  0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_dedicated,   qc_num_ifl_dedicated,	ATTR_UNDEF,	    qc_num_cpu_dedicated, 0)) ||
-			    (rc = qc_verify(hdl, qc_num_cp_shared,      qc_num_ifl_shared,	ATTR_UNDEF,	    qc_num_cpu_shared,	  0)) ||
+			if ((rc = qc_verify(hdl, qc_num_cpu_dedicated,  qc_num_cpu_shared,	ATTR_UNDEF,	    qc_num_cpu_total,	  1)) ||
+			    (rc = qc_verify(hdl, qc_num_cpu_configured, qc_num_cpu_reserved,	qc_num_cpu_standby, qc_num_cpu_total,	  1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_total,       qc_num_ifl_total,	ATTR_UNDEF,	    qc_num_cpu_total,	  1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_dedicated,   qc_num_ifl_dedicated,	ATTR_UNDEF,	    qc_num_cpu_dedicated, 1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_shared,      qc_num_ifl_shared,	ATTR_UNDEF,	    qc_num_cpu_shared,	  1)) ||
 			    (rc = qc_verify(hdl, qc_num_cp_dedicated,   qc_num_cp_shared,	ATTR_UNDEF,	    qc_num_cp_total,	  1)) ||
 			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,  qc_num_ifl_shared,	ATTR_UNDEF,	    qc_num_ifl_total,	  1)))
 				goto out;
 			break;
+		case QC_LAYER_TYPE_KVM_HYPERVISOR:
+			if ((rc = qc_verify(hdl, qc_num_cpu_shared, 	qc_num_cpu_dedicated,	ATTR_UNDEF,	    qc_num_cpu_total, 1)) ||
+			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,	qc_num_ifl_shared,	ATTR_UNDEF,	    qc_num_ifl_total, 1)) ||
+			    (rc = qc_verify(hdl, qc_num_cp_dedicated,	qc_num_cp_shared,	ATTR_UNDEF,	    qc_num_cp_total, 1)))
+				goto out;
+			break;
 		case QC_LAYER_TYPE_KVM_GUEST:
-			if ((rc = qc_verify(hdl, qc_num_cpu_configured, qc_num_cpu_reserved,	qc_num_cpu_standby, qc_num_cpu_total, 0)))
+			if ((rc = qc_verify(hdl, qc_num_cpu_configured, qc_num_cpu_reserved,	qc_num_cpu_standby, qc_num_cpu_total,	   1)) ||
+			    (rc = qc_verify(hdl, qc_num_cpu_shared, 	qc_num_cpu_dedicated,	ATTR_UNDEF,	    qc_num_cpu_total,	   1)) ||
+			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,	qc_num_ifl_shared,	ATTR_UNDEF,	    qc_num_ifl_total,	   1)) ||
+			    (rc = qc_verify(hdl, qc_num_ifl_dedicated,	ATTR_UNDEF,		ATTR_UNDEF,	    0,			   1)) ||
+			    (rc = qc_verify(hdl, qc_num_ifl_shared,	ATTR_UNDEF,		ATTR_UNDEF,	    qc_num_cpu_configured, 1)))
 				goto out;
 			break;
 		default: break;
@@ -408,9 +452,108 @@ out:
 	return rc;
 }
 
-static void *_qc_open(struct qc_handle *hdl, iconv_t *cd, int *rc) {
+static int qc_copy_attr_value(struct qc_handle *tgt, struct qc_handle *src, enum qc_attr_id id) {
+	int *i = qc_get_attr_value_int(src, id);
+
+	return i ? qc_set_attr_int(tgt, id, *i, ATTR_SRC_POSTPROC) : 0;
+}
+
+static int qc_post_process_KVM_host(struct qc_handle *hdl) {
+	struct qc_handle *parent = qc_get_prev_handle(hdl);
+	int *num_conf, rc;
+
+	qc_debug(hdl, "Fill KVM host layer\n");
+	qc_debug_indent_inc();
+	// We can copy most information from the parent with few exceptions
+	if (*(int *)(parent->layer) == QC_LAYER_TYPE_KVM_GUEST)
+		// KVM guests ain't got no CPs
+		rc = qc_set_attr_int(hdl, qc_num_cp_total, 0, ATTR_SRC_POSTPROC) ||
+		     qc_set_attr_int(hdl, qc_num_cp_dedicated, 0, ATTR_SRC_POSTPROC) ||
+		     qc_set_attr_int(hdl, qc_num_cp_shared, 0, ATTR_SRC_POSTPROC);
+	else
+		rc = qc_copy_attr_value(hdl, parent, qc_num_cp_total) ||
+		     qc_copy_attr_value(hdl, parent, qc_num_cp_dedicated) ||
+		     qc_copy_attr_value(hdl, parent, qc_num_cp_shared);
+	// only parent layer's configured CPUs are available to the host
+	num_conf = qc_get_attr_value_int(parent, qc_num_cpu_configured);
+	rc |= !num_conf ||
+	      qc_set_attr_int(hdl, qc_num_cpu_total, *num_conf, ATTR_SRC_POSTPROC) ||
+	      qc_copy_attr_value(hdl, parent, qc_num_cpu_dedicated) ||
+	      qc_copy_attr_value(hdl, parent, qc_num_cpu_shared) ||
+	      qc_copy_attr_value(hdl, parent, qc_num_ifl_total) ||
+	      qc_copy_attr_value(hdl, parent, qc_num_ifl_dedicated) ||
+	      qc_copy_attr_value(hdl, parent, qc_num_ifl_shared);
+	qc_debug_indent_dec();
+
+	return rc;
+}
+
+static int qc_post_process_KVM_guest(struct qc_handle *hdl) {
+	struct qc_handle *parent = qc_get_prev_handle(hdl);
+	int rc = 0, *num_conf, *num_CPs, *num_IFLs;
+
+	qc_debug(hdl, "Fill KVM guest layer\n");
+	qc_debug_indent_inc();
+	// KVM guests support IFLs only - all configured CPUs (comes from sysinfo!) are treated as IFLs
+	num_conf = qc_get_attr_value_int(hdl, qc_num_cpu_configured);
+	if (qc_set_attr_int(hdl, qc_num_ifl_total, *num_conf, ATTR_SRC_SYSINFO) ||
+	    qc_set_attr_int(hdl, qc_num_ifl_dedicated, 0, ATTR_SRC_SYSINFO) ||
+	    qc_set_attr_int(hdl, qc_num_ifl_shared, *num_conf, ATTR_SRC_SYSINFO) ||
+	    qc_set_attr_int(hdl, qc_num_cpu_dedicated, 0, ATTR_SRC_SYSINFO) ||
+	    qc_set_attr_int(hdl, qc_num_cpu_shared, *num_conf, ATTR_SRC_SYSINFO)) {
+		rc = -2;
+		goto out;
+	}
+
+	num_CPs = qc_get_attr_value_int(parent, qc_num_cp_total);
+	num_IFLs = qc_get_attr_value_int(parent, qc_num_ifl_total);
+	if (num_CPs && num_IFLs) {
+		if (*num_CPs > 0 && *num_IFLs > 0) {
+			qc_debug(hdl, "Warning: KVM guest running on a mixed-mode host!\n");
+			goto out;	// not an error
+		}
+		if (qc_set_attr_int(hdl, qc_ifl_dispatch_type, *num_IFLs > 0 ? 3 : 0, ATTR_SRC_POSTPROC)) {
+			rc = -3;
+			goto out;
+		}
+	}
+
+out:
+	qc_debug_indent_dec();
+
+	return rc;
+}
+
+static int qc_post_processing(struct qc_handle *hdl) {
+	qc_debug(hdl, "Post processing: Fill KVM layers\n");
+	qc_debug_indent_inc();
+	for (; hdl; hdl = hdl->next) {
+		switch(*(int *)(hdl->layer)) {
+		case QC_LAYER_TYPE_KVM_HYPERVISOR:
+			if (qc_post_process_KVM_host(hdl))
+				goto fail;
+			break;
+		case QC_LAYER_TYPE_KVM_GUEST:
+			if (qc_post_process_KVM_guest(hdl))
+				goto fail;
+			break;
+		default:
+			break;
+		}
+	}
+	qc_debug_indent_dec();
+
+	return 0;
+
+fail:
+	qc_debug_indent_dec();
+
+	return -1;
+}
+
+static void *_qc_open(struct qc_handle *hdl, int *rc) {
 	// sysinfo needs to be handled first, or our LGM check later on will have loopholes
-	struct qc_data_src *src, *sources[] = {&sysinfo, &hypfs, &sthyi, NULL};
+	struct qc_data_src *src, *sources[] = {&sysinfo, &ocf, &hypfs, &sthyi, NULL};
 	struct qc_handle *lparhdl;
 	int i;
 
@@ -439,7 +582,7 @@ static void *_qc_open(struct qc_handle *hdl, iconv_t *cd, int *rc) {
 	// process data sources
 	for (i = 0; (src = sources[i]) != NULL; i++) {
 		// Return values >0 will be left as is and passed back to caller
-		if ((*rc = src->process(hdl, cd, src->priv)) < 0) {
+		if ((*rc = src->process(hdl, src->priv)) < 0) {
 			*rc = -3;	// match errors to a value that we can identify
 			goto out;
 		}
@@ -447,6 +590,11 @@ static void *_qc_open(struct qc_handle *hdl, iconv_t *cd, int *rc) {
 			goto out;
 	}
 
+	if (qc_post_processing(hdl)) {
+		*rc = -4;
+		goto out;
+	}
+
 	if (qc_dbg_level > 0) {
 		qc_debug(hdl, "Final layers overview:\n");
 		qc_debug_indent_inc();
@@ -478,10 +626,9 @@ out:
 }
 
 void *qc_open(int *rc) {
-	int cd_init = 0, consistency_check_requested = 0, i;
 	struct qc_handle *hdl = NULL;
 	char *s, *end;
-	iconv_t cd;
+	int i;
 
 	*rc = 0;
 	if (qc_debug_init()) {
@@ -491,18 +638,20 @@ void *qc_open(int *rc) {
 	qc_debug(hdl, "qc_open()\n");
 	qc_debug_indent_inc();
 
-	cd = iconv_open("ISO8859-1", "IBM-1047");
-	if (cd == (iconv_t)-1) {
-		qc_debug(hdl, "Error: iconv setup failed: %s\n", strerror(errno));
-		*rc = -2;
-		goto out;
+	if (qc_cd == (iconv_t)-1) {
+		qc_debug(hdl, "Initialize iconv\n");
+		qc_cd = iconv_open("ISO8859-1", "IBM-1047");
+		if (qc_cd == (iconv_t)-1) {
+			qc_debug(hdl, "Error: iconv setup failed: %s\n", strerror(errno));
+			*rc = -2;
+			goto out;
+		}
 	}
-	cd_init = 1;
 
 	if ((s = getenv("QC_CHECK_CONSISTENCY")) != NULL) {
-		consistency_check_requested = strtol(s, &end, 10);
-		if (end == s || consistency_check_requested < 0)
-			qc_dbg_level = 0;
+		qc_consistency_check_requested = strtol(s, &end, 10);
+		if (end == s || qc_consistency_check_requested < 0)
+			qc_consistency_check_requested = 0;
 	}
 
 	/* Since we retrieve data from multiple sources, CPU hotplugging provides a chance for
@@ -510,11 +659,11 @@ void *qc_open(int *rc) {
 	 * giving up. */
 	for (i = 0; i < 3; ++i) {
 		if (i > 0) {
-			qc_debug(hdl, "Warning: Sanity check failed, retry %d\n", i);
+			qc_debug(hdl, "Warning: Consistency check failed, retry %d\n", i);
 			qc_hdl_reinit(hdl);
 		}
-		hdl = _qc_open(hdl, &cd, rc);
-		if (*rc	|| (consistency_check_requested && (*rc = qc_consistency_check(hdl)) <= 0))
+		hdl = _qc_open(hdl, rc);
+		if (*rc	|| ((*rc = qc_consistency_check(hdl)) <= 0))
 			break;
 	}
 	if (*rc > 0) {
@@ -522,8 +671,6 @@ void *qc_open(int *rc) {
 	}
 
 out:
-	if (cd_init)
-		iconv_close(cd);
 	qc_debug(hdl, "Return %p, rc=%d\n", *rc ? NULL : hdl, *rc);
 	qc_debug_indent_dec();
 	if (*rc) {
@@ -549,7 +696,7 @@ static struct qc_handle *qc_get_layer_handle(void *config, int layer) {
 }
 
 static int qc_is_attr_id_valid(enum qc_attr_id id) {
-	return id >= 0 && id <= qc_type;
+	return id <= qc_prorated_core_time;
 }
 
 int qc_get_attribute_string(void *cfg, enum qc_attr_id id, int layer, const char **value) {
